@@ -663,6 +663,10 @@ fs_finalize (GObject *object)
 		g_hash_table_unref (priv->mtime_cache);
 	}
 
+	if (priv->iri_cache) {
+		g_hash_table_unref (priv->iri_cache);
+	}
+
 	G_OBJECT_CLASS (tracker_miner_fs_parent_class)->finalize (object);
 }
 
@@ -987,6 +991,8 @@ sparql_update_cb (GObject      *object,
 				 */
 				g_hash_table_insert (fs->private->iri_cache, g_object_ref (data->file), NULL);
 			}
+
+			g_object_unref (parent);
 		}
 	}
 
@@ -1119,22 +1125,48 @@ cache_query_cb (GObject	     *object,
 	}
 }
 
+static gboolean
+file_is_crawl_directory (TrackerMinerFS *fs,
+                         GFile          *file)
+{
+	GList *dirs;
+
+	/* Check whether file is a crawl directory itself */
+	dirs = fs->private->config_directories;
+
+	while (dirs) {
+		DirectoryData *data;
+
+		data = dirs->data;
+		dirs = dirs->next;
+
+		if (g_file_equal (data->file, file)) {
+			return TRUE;
+		}
+	}
+
+	return FALSE;
+}
+
 static void
 ensure_iri_cache (TrackerMinerFS *fs,
-                  GFile          *parent)
+                  GFile          *file)
 {
 	gchar *query, *uri;
 	CacheQueryData data;
+	GFile *parent;
 
 	g_hash_table_remove_all (fs->private->iri_cache);
 
+	parent = g_file_get_parent (file);
 	uri = g_file_get_uri (parent);
 
 	g_debug ("Generating IRI cache for folder: %s", uri);
 
-	query = g_strdup_printf ("SELECT ?uri ?u { "
-	                         "  ?u nie:url ?uri . "
-	                         "  FILTER (tracker:uri-is-parent (\"%s\", ?uri)) "
+	query = g_strdup_printf ("SELECT ?url ?u { "
+	                         "  ?u nfo:belongsToContainer ?p ; "
+	                         "     nie:url ?url . "
+	                         "  ?p nie:url \"%s\" "
 	                         "}",
 	                         uri);
 	g_free (uri);
@@ -1152,6 +1184,19 @@ ensure_iri_cache (TrackerMinerFS *fs,
 
 	g_main_loop_unref (data.main_loop);
 	g_hash_table_unref (data.values);
+
+	if (g_hash_table_size (data.values) == 0 &&
+	    file_is_crawl_directory (fs, file)) {
+		gchar *query_iri;
+
+		if (item_query_exists (fs, file, &query_iri, NULL)) {
+			g_hash_table_insert (data.values,
+			                     g_object_ref (file), query_iri);
+		}
+	}
+
+	g_object_unref (parent);
+	g_free (query);
 }
 
 static const gchar *
@@ -1177,6 +1222,7 @@ iri_cache_lookup (TrackerMinerFS *fs,
 		if (item_query_exists (fs, file, &query_iri, NULL)) {
 			g_hash_table_insert (fs->private->iri_cache,
 			                     g_object_ref (file), query_iri);
+			iri = query_iri;
 		} else {
 			g_hash_table_remove (fs->private->iri_cache, file);
 			iri = NULL;
@@ -1338,7 +1384,7 @@ item_add_or_update (TrackerMinerFS *fs,
 				fs->private->current_parent_urn = NULL;
 			}
 
-			ensure_iri_cache (fs, parent);
+			ensure_iri_cache (fs, file);
 		}
 
 		parent_urn = fs->private->current_parent_urn;
@@ -1606,12 +1652,13 @@ item_move (TrackerMinerFS *fs,
            GFile          *file,
            GFile          *source_file)
 {
-	gchar     *uri, *source_uri, *escaped_filename;
+	gchar     *uri, *source_uri;
 	GFileInfo *file_info;
 	GString   *sparql;
 	RecursiveMoveData move_data;
 	ProcessData *data;
 	gchar *source_iri;
+	gchar *display_name;
 	gboolean source_exists;
 
 	iri_cache_invalidate (fs, file);
@@ -1700,7 +1747,7 @@ item_move (TrackerMinerFS *fs,
 	                        "} ",
 	                        source_iri, source_iri, source_iri);
 
-	escaped_filename = g_strescape (g_file_info_get_display_name (file_info), NULL);
+	display_name = tracker_sparql_escape (g_file_info_get_display_name (file_info));
 
 	g_string_append_printf (sparql,
 	                        "INSERT INTO <%s> {"
@@ -1709,10 +1756,10 @@ item_move (TrackerMinerFS *fs,
 	                        "       nie:isStoredAs <%s> "
 	                        "} ",
 	                        source_iri, source_iri,
-	                        escaped_filename, uri,
+	                        display_name, uri,
 	                        source_iri);
 
-	g_free (escaped_filename);
+	g_free (display_name);
 
 	move_data.main_loop = g_main_loop_new (NULL, FALSE);
 	move_data.sparql = sparql;
@@ -2182,11 +2229,11 @@ ensure_mtime_cache (TrackerMinerFS *fs,
 
 	g_debug ("Generating mtime cache for folder: %s", uri);
 
-	query = g_strdup_printf ("SELECT ?uri nfo:fileLastModified (?u) { "
-	                         "  ?u nie:url ?uri . "
-	                         "  FILTER (tracker:uri-is-parent (\"%s\", ?uri)) "
-	                         "}",
-	                         uri);
+	query = g_strdup_printf ("SELECT ?url ?last { ?u nfo:belongsToContainer ?p ; "
+	                                                "nie:url ?url ; "
+	                                                "nfo:fileLastModified ?last . "
+	                                             "?p nie:url \"%s\" }", uri);
+
 	g_free (uri);
 
 	data.main_loop = g_main_loop_new (NULL, FALSE);
@@ -2197,9 +2244,35 @@ ensure_mtime_cache (TrackerMinerFS *fs,
 	                              NULL,
 	                              cache_query_cb,
 	                              &data);
+	g_free (query);
 
 	g_main_loop_run (data.main_loop);
 
+	if (g_hash_table_size (data.values) == 0 &&
+	    file_is_crawl_directory (fs, file)) {
+		/* File is a crawl directory itself, query its mtime directly */
+		uri = g_file_get_uri (file);
+
+		g_debug ("Folder %s is a crawl directory, generating mtime cache for it", uri);
+
+		query = g_strdup_printf ("SELECT ?url ?last "
+		                         "WHERE { "
+		                         "  ?u nfo:fileLastModified ?last ; "
+		                         "     nie:url ?url ; "
+		                         "     nie:url \"%s\" "
+		                         "}", uri);
+		g_free (uri);
+
+		tracker_miner_execute_sparql (TRACKER_MINER (fs),
+		                              query,
+		                              NULL,
+		                              cache_query_cb,
+		                              &data);
+		g_free (query);
+
+		g_main_loop_run (data.main_loop);
+	}
+
 	g_main_loop_unref (data.main_loop);
 	g_hash_table_unref (data.values);
 }
@@ -2697,7 +2770,7 @@ static gboolean
 crawl_directories_cb (gpointer user_data)
 {
 	TrackerMinerFS *fs = user_data;
-	gchar *path;
+	gchar *path, *path_utf8;
 	gchar *str;
 
 	if (fs->private->current_directory) {
@@ -2724,12 +2797,15 @@ crawl_directories_cb (gpointer user_data)
 	                                          fs->private->current_directory);
 
 	path = g_file_get_path (fs->private->current_directory->file);
+	path_utf8 = g_filename_to_utf8 (path, -1, NULL, NULL, NULL);
+	g_free (path);
 
 	if (fs->private->current_directory->recurse) {
-		str = g_strdup_printf ("Crawling recursively directory '%s'", path);
+		str = g_strdup_printf ("Crawling recursively directory '%s'", path_utf8);
 	} else {
-		str = g_strdup_printf ("Crawling single directory '%s'", path);
+		str = g_strdup_printf ("Crawling single directory '%s'", path_utf8);
 	}
+	g_free (path_utf8);
 
 	g_message ("%s", str);
 
@@ -2739,7 +2815,6 @@ crawl_directories_cb (gpointer user_data)
 	              "status", str,
 	              NULL);
 	g_free (str);
-	g_free (path);
 
 	if (tracker_crawler_start (fs->private->crawler,
 	                           fs->private->current_directory->file,
