@@ -1,6 +1,5 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2009, Nokia
+ * Copyright (C) 2009, Nokia <ivan.frade@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -36,7 +35,7 @@ typedef void (* TrackerStoreCommitCallback)       (gpointer         user_data);
 typedef void (* TrackerStoreTurtleCallback)       (GError          *error,
                                                    gpointer         user_data);
 
-void         tracker_store_init                   (gboolean load_journal);
+void         tracker_store_init                   (void);
 void         tracker_store_shutdown               (void);
 void         tracker_store_queue_commit           (TrackerStoreCommitCallback callback,
                                                    const gchar   *client_id,
@@ -61,8 +60,6 @@ tracker_store_sparql_query           (const gchar   *sparql,
 
 guint        tracker_store_get_queue_size         (void);
 
-void         tracker_store_play_journal           (void);
-void         tracker_store_flush_journal          (void);
 void         tracker_store_unreg_batches          (const gchar   *client_id);
 
 G_END_DECLS
